@@ -16,6 +16,7 @@
 
 (define-module (aa-tree)
   #:use-module (oop goops)
+  #:use-module (srfi srfi-1)
   #:export (aa-delete
             aa-insert
             aa-map
@@ -23,13 +24,21 @@
             aa-map-reverse
             aa-map-values
             aa-search
-            aa-to-list))
+            aa-to-list
+            list-to-aa-tree))
 
 (define-method (< (a <string>) . rest) (apply string<? a rest))
 
-(define-method (< (a <symbol>) . rest) (apply string<? (symbol->string a) (map symbol->string rest)))
-
+(define-method (< (a <symbol>) . rest)
+  (apply string<? (symbol->string a) (map symbol->string rest)))
 
+(define-method (< (a <keyword>) . rest)
+  (apply string<?
+         (symbol->string (keyword->symbol a))
+         (map
+          (lambda (x)
+            (symbol->string (keyword->symbol x)))
+          rest)))
 
 ;; aa tree structure is
 ;;
@@ -172,6 +181,13 @@
 ;; The key parameter may be a list of keys, in which case the function
 ;; will do a nested delete.
 (define (aa-delete tree k)
+  "Performs a non-mutating delete on tree. Deletes the node in tree
+which has key key. If there is no such node, it simply returns
+tree. O(log n) worst case efficiency. If a list of keys is provided
+instead of a single key, aa-delete will attempt a nested tree delete,
+starting the search with the first key in the list. If a nested tree
+of some required level does not exist, it will simply return the
+original tree unmodified."
   (if (list? k) (aa-delete-nested tree k)
       (let* ((t1
               (skew
@@ -303,3 +319,16 @@
            (aa-search tree (car key-lst))))
       (if (nil? search-result) #nil
           (aa-search-nested (car search-result) (cdr key-lst)))))))
+
+(define (list-to-aa-tree init-tree lst)
+  (cond
+   ((null? lst) init-tree)
+   ((null? (cdr lst)) (throw 'missing-corresponding-value))
+   (else (list-to-aa-tree
+          (aa-insert init-tree
+                     (first lst)
+                     (cond
+                      ((not (list? (second lst))) (second lst))
+                      ((equal? '_ (first (second lst))) (cdr (second lst)))
+                      (else (list-to-aa-tree #nil (second lst)))))
+          (drop lst 2)))))
